"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("./types");
exports.diff = function (left, right, options, currentPath) {
    if (options === void 0) { options = []; }
    if (currentPath === void 0) { currentPath = ''; }
    options = options.map(function (x) { return (__assign(__assign({}, defaultOption), x)); });
    var option = getOption(options, currentPath);
    if (option.ignore) {
        return [];
    }
    if (isUndefined(left) && isUndefined(right)) {
        return [];
    }
    if (isUndefined(left)) {
        return [{ path: currentPath, type: types_1.DiffResultTypeEnum.Added, right: right }];
    }
    if (isUndefined(right)) {
        return [{ path: currentPath, type: types_1.DiffResultTypeEnum.Deleted, left: left }];
    }
    if (isPrimitiveType(left)) {
        return left === right
            ? []
            : [{ path: currentPath, type: types_1.DiffResultTypeEnum.Editted, left: left, right: right }];
    }
    if (isObject(left)) {
        right = right || {};
        var aKeys_1 = Object.keys(left);
        var keysOnlyBHas = Object.keys(right).filter(function (key) { return !aKeys_1.includes(key); });
        var mismatches = aKeys_1.reduce(function (prev, key) { return __spreadArrays(prev, exports.diff(left[key], right[key], options, createPath(currentPath, key))); }, []);
        var missings = keysOnlyBHas.reduce(function (prev, key) { return __spreadArrays(prev, exports.diff(left[key], right[key], options, createPath(currentPath, key))); }, []);
        return __spreadArrays(mismatches, missings);
    }
    if (isArray(left)) {
        right = right || [];
        var _a = __assign(__assign({}, defaultOption), (option || {})).array, arrayOptions = _a === void 0 ? defaultArrayOption : _a;
        return arrayOptions.orderSensitive
            ? diffArrayWithSensitiveOrder(left, right)
            : diffArrayWithInsensitiveOrder(left, right, arrayOptions);
    }
    return [];
    function diffArrayWithInsensitiveOrder(x1, x2, _a) {
        var _b = _a.getKey, getKey = _b === void 0 ? defaultGetKey : _b, _c = _a.ignoreUndefinedKey, ignoreUndefinedKey = _c === void 0 ? true : _c;
        var filter = ignoreUndefinedKey ? function (x) { return !isUndefined(x); } : function () { return true; };
        var aKeys = x1.map(getKey).filter(filter);
        var keysOnlyBHas = x2
            .map(getKey)
            .filter(function (key) { return !aKeys.includes(key); })
            .filter(filter);
        var findByKey = function (array, key) {
            return array.find(function (item) { return getKey(item) === key; });
        };
        var mismatches = aKeys.reduce(function (prev, key) { return __spreadArrays(prev, exports.diff(findByKey(x1, key), findByKey(x2, key), options, createPathToNonOrderedArray(currentPath, key))); }, []);
        var missings = keysOnlyBHas.reduce(function (prev, key) { return __spreadArrays(prev, exports.diff(findByKey(x1, key), findByKey(x2, key), options, createPathToNonOrderedArray(currentPath, key))); }, []);
        return __spreadArrays(mismatches, missings);
    }
    function diffArrayWithSensitiveOrder(x1, x2) {
        var maxLength = Math.max(x1.length, x2.length);
        return range(maxLength).reduce(function (prev, index) { return __spreadArrays(prev, exports.diff(x1[index], x2[index], options, createPathToOrderedArray(currentPath, index))); }, []);
    }
};
var defaultGetKey = function (x) { return x; };
var defaultArrayOption = {
    getKey: defaultGetKey,
    orderSensitive: true
};
var defaultOption = {
    path: '',
    ignore: false
};
var isUndefined = function (x) { return typeof x === 'undefined'; };
var isPrimitiveType = function (x) {
    return ['string', 'number', 'boolean'].includes(typeof x);
};
var isArray = function (x) { return Array.isArray(x); };
var isObject = function (x) { return typeof x === 'object' && !isArray(x); };
var createPath = function (currentLoc, name) {
    return !currentLoc ? name : currentLoc + "." + name;
};
var createPathToOrderedArray = function (currentLoc, index) {
    return currentLoc + "[" + index + "]";
};
var createPathToNonOrderedArray = function (currentLoc, key) {
    return currentLoc + "(" + key + ")";
};
var range = function (max) { return Array.from(Array(max).keys()); };
var getOption = function (options, loc) {
    if (options === void 0) { options = []; }
    var filterMatched = function (_a) {
        var path = _a.path;
        if (typeof path === 'string') {
            return loc === path;
        }
        if (path instanceof RegExp) {
            return path.test(loc);
        }
        return false;
    };
    return options
        .filter(filterMatched)
        .reduce(function (aggr, _a) {
        var path = _a.path, option = __rest(_a, ["path"]);
        return (__assign(__assign({}, aggr), option));
    }, defaultOption);
};
